from __future__ import annotations

from typing import TYPE_CHECKING, Tuple

from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from edc_consent.utils import get_consent_model_cls

if TYPE_CHECKING:
    from intecomm_screening.models import SubjectScreening


def get_add_or_change_consent_url(
    obj: SubjectScreening, next_url_name: str | None = None
) -> Tuple[str | None, str | None, str | None]:
    add_consent_url = None
    change_consent_url = None
    subject_identifier = None

    next_url_name = (
        next_url_name or "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
    )

    try:
        subject_consent = get_consent_model_cls().objects.get(
            screening_identifier=obj.screening_identifier
        )
    except ObjectDoesNotExist:
        url = reverse("intecomm_consent_admin:intecomm_consent_subjectconsent_add")
        add_consent_url = (
            f"{url}?next={next_url_name}"
            f"&screening_identifier={obj.screening_identifier}"
            f"&identity={obj.hospital_identifier}"
            f"&initials={obj.initials}"
            f"&site={obj.site.id}"
            f"&gender={obj.gender}"
            f"&age_in_years={obj.age_in_years}"
            f"&legal_name={obj.legal_name}"
            f"&familiar_name={obj.familiar_name}"
        )
    else:
        subject_identifier = subject_consent.subject_identifier
        url = reverse(
            "intecomm_consent_admin:intecomm_consent_subjectconsent_change",
            args=(subject_consent.id,),
        )
        change_consent_url = f"{url}?next={next_url_name}"
    return add_consent_url, change_consent_url, subject_identifier



from django.contrib import admin
from edc_model_admin.mixins import TabularInlineMixin
from edc_rx.modeladmin_mixins import DrugRefillAdminMixin, DrugSupplyInlineMixin

from ..admin_site import intecomm_subject_admin
from ..forms import DrugRefillHtnForm, DrugSupplyHtnForm
from ..models import DrugRefillHtn, DrugSupplyHtn
from .modeladmin_mixins import CrfModelAdmin


class DrugSupplyHtnInline(DrugSupplyInlineMixin, TabularInlineMixin, admin.TabularInline):
    model = DrugSupplyHtn
    form = DrugSupplyHtnForm


@admin.register(DrugRefillHtn, site=intecomm_subject_admin)
class DrugRefillHtnAdmin(DrugRefillAdminMixin, CrfModelAdmin):
    form = DrugRefillHtnForm
    inlines = [DrugSupplyHtnInline]



from django import forms
from django.conf import settings
from edc_appointment.exceptions import AppointmentWindowError
from edc_appointment.utils import raise_on_appt_datetime_not_in_window
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_utils import convert_php_dateformat, to_utc
from intecomm_form_validators.subject import NextAppointmentFormValidator

from ..models import NextAppointment


class NextAppointmentForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = NextAppointmentFormValidator

    def clean(self):
        cleaned_data = super().clean()
        self.validate_best_visit_code()
        return cleaned_data

    def validate_best_visit_code(self):
        if appt_date := self.cleaned_data.get("appt_date"):
            subject_visit = self.cleaned_data.get("subject_visit")
            appointment = subject_visit.appointment.next
            while appointment:
                if appt_date <= to_utc(appointment.appt_datetime).date():
                    break
                appointment = appointment.next
            # is after appointment.previous and before appointment
            try:
                raise_on_appt_datetime_not_in_window(appointment)
            except AppointmentWindowError:
                appointment = appointment.previous
            if self.cleaned_data.get("best_visit_code") != appointment.visit_code:
                date_format = convert_php_dateformat(settings.SHORT_DATE_FORMAT)
                raise forms.ValidationError(
                    {
                        "best_visit_code": (
                            f"Expected {appointment.visit_code} "
                            f"using {appt_date.strftime(date_format)} from above."
                        )
                    }
                )

    class Meta:
        model = NextAppointment
        fields = "__all__"



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin
from edc_sites.modelform_mixins import SiteModelFormMixin
from edc_sites.widgets import SiteField
from intecomm_form_validators import SubjectScreeningFormValidator

from ..models import SubjectScreening


class SubjectScreeningForm(
    AlreadyConsentedFormMixin, SiteModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    form_validator_cls = SubjectScreeningFormValidator

    site = SiteField()

    class Meta:
        model = SubjectScreening
        fields = "__all__"
        labels = {
            "consent_ability": "Is the patient able and willing to give informed consent."
        }
        widgets = {
            "legal_name": forms.TextInput(attrs={"readonly": "readonly"}),
            "familiar_name": forms.TextInput(attrs={"readonly": "readonly"}),
            "initials": forms.TextInput(attrs={"readonly": "readonly"}),
            "age_in_years": forms.TextInput(attrs={"readonly": "readonly"}),
            "hospital_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from .consent_refusal import ConsentRefusal
from .health_facility import HealthFacility
from .health_talk_log import HealthTalkLog
from .identifier_format import IdenfifierFormat
from .patient_call import PatientCall
from .patient_log import PatientLog
from .patient_log_report_print_history import PatientLogReportPrintHistory
from .proxy_models import PatientGroup, Site
from .signals import (
    patient_call_on_post_delete,
    patient_call_on_post_save,
    patientlog_on_pre_delete,
    subjectscreening_on_post_delete,
    subjectscreening_on_pre_delete,
    update_subjectscreening_on_post_save,
)
from .subject_screening import SubjectScreening



from django.contrib import admin

from ..admin_site import intecomm_group_admin
from ..models import PatientLog


@admin.register(PatientLog, site=intecomm_group_admin)
class PatientLogAdmin(admin.ModelAdmin):
    """Registered again for the autocomplete field"""

    search_fields = ("name",)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..forms import HtnInitialReviewForm
from ..models import HtnInitialReview
from .modeladmin_mixins import CrfModelAdmin


@admin.register(HtnInitialReview, site=intecomm_subject_admin)
class HtnInitialReviewAdmin(CrfModelAdmin):
    form = HtnInitialReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Hypertension diagnosis",
            {
                "fields": (
                    "dx_date",
                    "dx_ago",
                )
            },
        ),
        (
            "Hypertension treatment",
            {
                "fields": (
                    "managed_by",
                    "managed_by_other",
                    "rx_init_date",
                    "rx_init_ago",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        # "managed_by": admin.VERTICAL,
    }

    filter_horizontal = ["managed_by"]

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj=obj, **kwargs)
        form = self.replace_label_text(form, "diagnosis_label", self.model.diagnosis_label)
        return form



from django import forms
from intecomm_form_validators.subject import HealthEconomicsFormValidator

from ..models import HealthEconomics
from .mixins import CrfModelFormMixin


class HealthEconomicsForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HealthEconomicsFormValidator

    class Meta:
        model = HealthEconomics
        fields = "__all__"



from typing import Tuple

from django.contrib import admin
from edc_data_manager.modeladmin_mixins import DataManagerModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import intecomm_prn_admin
from ..models import OnScheduleInte


@admin.register(OnScheduleInte, site=intecomm_prn_admin)
class OnScheduleInteAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):
    instructions = None

    fields = ("subject_identifier", "onschedule_datetime")

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = ("subject_identifier", "dashboard", "onschedule_datetime")
        return custom_fields + tuple(
            f for f in list_display if f not in custom_fields + ("__str__",)
        )

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("onschedule_datetime",)
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        return (
            "subject_identifier",
            "onschedule_datetime",
        )

    def get_search_fields(self, request) -> Tuple[str, ...]:
        return tuple(set(super().get_search_fields(request) + ("subject_identifier",)))



from django.urls import reverse
from edc_subject_dashboard.views import SubjectDashboardView

from intecomm_group.models import PatientGroup


class DashboardView(SubjectDashboardView):
    consent_model = "intecomm_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "intecomm_subject.subjectvisit"
    history_button_label = "Audit"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        group = PatientGroup.objects.get(
            group_identifier=context.get("consent").object.group_identifier
        )
        url = reverse("intecomm_group_admin:intecomm_group_patientgroup_changelist")
        url = f"{url}?q={group.name}"

        context.update(
            subject_listboard_url="screen_group_url",
            group_identifier=group.group_identifier,
            group_name=group.name,
            patient_group_url=url,
        )
        return context
